# --------------------------------------------------------------
# TASKMAP UNIT TESTS
# To be used in the Jenkins build pipeline for VITO-PyREST
# --------------------------------------------------------------
# IMPORTS AND LIBRARIES
# --------------------------------------------------------------
#from __future__ import absolute_import
import unittest
from db.wigldap import WigLDAP
from db.database import WigFields
from ..utils.taskmap import Taskmap
from os.path import join
import requests
import os
import binascii
import json

import logging


# --------------------------------------------------------------
# VARIABLES
# --------------------------------------------------------------
log = logging.getLogger()
ldap = WigLDAP()
# --------------------------------------------------------------
# MAIN TEST FUNCTIONS
# --------------------------------------------------------------
def main_test(login, workdir):
    wdir = join(workdir, str(binascii.hexlify(os.urandom(16))).replace('\'', ''))  # Generates a random foldername
    result_list = []
    src_fields = fetch_fields(login)

    for field in src_fields:
        log.debug("Processing field %s " % field["metadata"]["fieldName"])
        geometry = field["metadata"]["geometry"]

        jsondata = {}
        jsondata['timestamp'] = ''
        jsondata['geometry'] = geometry
        jsondata['stepvalue'] = 0
        jsondata['maxvalue'] = 0
        jsondata['minvalue'] = 0

        jsonstr = json.dumps(jsondata)

        try:
            headers = {'Content-type': 'application/json'}
            old_json = requests.post('http://pyrest-dev.vgt.vito.be:8080/taskmap/generate/shadow', data=jsonstr, headers=headers, timeout=5).content
        except Exception as e:
            log.error("Test failed: {0}".format(e))
            return False



        new_json = Taskmap.render_shadowmap_from_geojson(jsondata, wdir)
        #print("OLD: ========================================================= {0}".format(old_json))
        #print("NEW: ========================================================= {0}".format(new_json))
        print(type(old_json))
        try:
            old_json_load = json.loads(old_json)
            new_json_load = json.loads(new_json)

            result_list.append(compare_results(old_json_load, new_json_load))
        except Exception as e:
            log.error("Test failed: {0}".format(e))




    if len(result_list) > 0:
        print(result_list)
        return all(result_list)
    else:
        return True


def compare_results(old_result, new_result):
    # Check the geometry of old_result, compare it with new_result
    old_features = old_result['map']['features']
    new_features = new_result['map']['features']
    old_array = []
    new_array = []
    for feature in old_features:
        print(feature['geometry'])
        featuredict = {'class': feature['properties']['classnr'], 'geometry': feature['geometry']}
        old_array.append(featuredict)
    print(old_array)
    for feature in new_features:
        print(feature['geometry'])
        featuredict = {'class': feature['properties']['classnr'], 'geometry': feature['geometry']}
        new_array.append(featuredict)
    print(old_array)

    if old_array == new_array:
        return True
    else:
        return False


def fetch_fields(useremail):
    # Retrieve user ID via LDAP
    uid = ldap.getInfoByMail(useremail)['uid'][0].decode('utf-8')
    # Retrieve the fields of the user
    src_fields = list(map(lambda x: x["_source"], WigFields.getParcelsByUser(uid)))
    print("Found %s parcels for user %s" % (len(src_fields), uid))

    return src_fields

if __name__ == '__main__':
    unittest.main()